var fs = require('fs');

var mesa = require('mesa');

function readFile(path, options) {
  const defer = Promise.defer();
  fs.readFile(path, options, function(err, result) {
    if (err) {
      return defer.reject(err);
    }
    defer.resolve(result);
  });
  return defer.promise;
}

function readdir(path) {
  const defer = Promise.defer();
  fs.readdir(path, function(err, result) {
    if (err) {
      return defer.reject(err);
    }
    defer.resolve(result);
  });
  return defer.promise;
}

function migrate(path, connection, isSync) {

  var schemaInfo = mesa
    .setConnection(connection)
    .table('schema_info')
    .allow(['version'])
    .returning('version');

  return Promise.resolve()
    .then(isSync ? syncDatabase : Function)
    .then(createSchemaInfoTableIfNotExists)
    .then(executeMigrations)
    .then(migrations => migrations.map(migration => 'Added ' + migration.description + ' to database.'));

  function syncDatabase() {
    var dropPath = __dirname + '/share/drop-all-tables.sql';
    return readFile(dropPath, 'utf8').then(schemaInfo.query.bind(schemaInfo));
  }

  function createSchemaInfoTableIfNotExists() {
    var sql = 'CREATE TABLE IF NOT EXISTS ' +
      'schema_info (version bigint NOT NULL UNIQUE);';
    return schemaInfo.query(sql);
  }

  function executeMigrations() {
    return Promise.all([
      getAvailableMigrations(),
      getAllAlreadyExecutedMigrations(),
    ])
    .then(promises => getNewMigrations(promises[0], promises[1]))
    .then(results => Promise.all(results.map(addSqlToMigration)))
    .then(results => Promise.all(results.map(executeMigration)));
  }

  function getAvailableMigrations() {
    return readdir(path)
      .then(filenames => filenames.map(parseSchemaFileName))
      .then(filenames => filenames.filter(isValidSchemaFile))
      .then(migrations => migrations.slice().sort(compareVersions));

    function compareVersions(a, b) {
      if (a.version < b.version) {
        return -1;
      }
      if (a.version > b.version) {
        return 1;
      }
      return 0;
    }

    function parseSchemaFileName(filename) {
      var date = filename.split('-').shift();
      var description = filename.substr(date.length + 1).split('.').shift();
      return {
        description: description,
        extension: filename.split('.').pop(),
        filename: filename,
        version: date,
      };
    }

    function isValidSchemaFile(schemaFile) {
      var isSqlExtension = schemaFile.extension === 'sql';
      var correctLength = schemaFile.version.toString().length === 14;
      return isSqlExtension && !isNaN(schemaFile.version) && correctLength;
    }
  }

  function getAllAlreadyExecutedMigrations() {
    return schemaInfo.find()
      .then(migrations => migrations.map(migration => migration.version));
  }

  function getNewMigrations(availableSchemas, executedVersions) {
    return availableSchemas.filter(function(schemaFile) {
      return executedVersions.indexOf(schemaFile.version.toString()) === -1;
    });
  }

  function addSqlToMigration(migration) {
    return readFile(path + '/' + migration.filename, 'utf8')
      .then(sql => Object.assign({sql: sql}, migration));
  }

  function executeMigration(migration) {
    console.log('executeMigration', JSON.stringify(migration));
    return schemaInfo.query(migration.sql)
      .then(function() {
        var preHook = '00000000000000';
        var postHook = '99999999999999';
        var isHook = preHook === migration.version || postHook === migration.version;
        return isHook ? null : schemaInfo.insert({version: migration.version});
      })
      .then(() => migration);
  }
}
module.exports = migrate;
