const assert = require('assert');
const fs = require('fs');

const pg = require('pg');
const mockFs = require('mock-fs');

const migrate = require('../migrate');

assert(process.env.DATABASE_URL, 'Should have a DATABASE_URL.');

describe('migrate', () => {

  var client;
  var done;

  beforeEach('get postgres connection', (beforeDone) => {
    pg.connect(process.env.DATABASE_URL, (err, pgClient, pgDone) => {
      client = pgClient;
      done = pgDone;
      beforeDone(err);
    });
  })

  beforeEach('reset database', () => {
    client.query('DROP TABLE IF EXISTS test, schema_info');
  });

  afterEach('end postgres connection', () => {
    done();
  });

  it('should migrate a schema',() => {
    mockFs({
      path: {
        '20000000000000-name.sql': 'CREATE TABLE test (value TEXT);',
        '20000000000001-name.sql': 'INSERT INTO test (value) VALUES (\'value\');',
      }
    });
    return migrate('path', client)
      .then(() => client.query('SELECT * FROM test'))
      .then(result => assert.deepEqual(result.rows, [{value: 'value'}]))
      .then(() => client.query('SELECT * FROM schema_info'))
      .then(result => assert.deepEqual(result.rows, [
        {version: '20000000000000'},
        {version: '20000000000001'}
      ]))
      .then(() => mockFs.restore());
  });

  it('should be idempotent', () => {
    mockFs({path: {
      '20000000000000-name.sql': 'CREATE TABLE test (value TEXT);'
    }});
    return migrate('path', client)
      .then(() => migrate('path', client))
      .then(() => mockFs.restore());
  });

  it('should allow hooks', () => {
    mockFs({path: {
      '00000000000000-prehook.sql': 'INSERT INTO test (value) VALUES (\'pre\')',
      '20000000000000-name.sql': 'INSERT INTO test (value) VALUES (\'value\')',
      '99999999999999-posthook.sql': 'INSERT INTO test (value) VALUES (\'post\')'
    }});
    return client.query('CREATE TABLE test (value TEXT)')
      .then(() => migrate('path', client))
      .then(() => migrate('path', client))
      .then(() => client.query('SELECT * FROM test'))
      .then(results => assert.deepEqual(results.rows, [
        {value: 'pre'},
        {value: 'value'},
        {value: 'post'},
        {value: 'pre'},
        {value: 'post'},
      ]))
      .then(() => mockFs.restore());
  });

  it('should allow to sync database', () => {
    var dropAllTables = fs.readFileSync(__dirname + '/../share/drop-all-tables.sql');
    mockFs({
      '/Users/engel/Documents/megrim/share/drop-all-tables.sql': dropAllTables,
      path: {
      '20000000000000-name.sql': 'CREATE TABLE test (value TEXT);'
    }});
    return migrate('path', client)
      .then(() => client.query('INSERT INTO test (value) VALUES (\'value\')'))
      .then(() => migrate('path', client, true))
      .then(() => client.query('SELECT * FROM test'))
      .then(result => assert.deepEqual(result.rows, []))
      .then(() => mockFs.restore());
  });
});
